@@ -10,7 +10,7 @@ use Types::Serialiser ();
 use XSLoader ();
 
 BEGIN {
-    $VERSION = '0.01_04';
+    $VERSION = '0.03_01';
     XSLoader::load();
 }
 
@@ -18,7 +18,13 @@ BEGIN {
 
 =head1 NAME
 
-TOML::XS - Parse L<TOML|https://toml.io> with XS
+TOML::XS - Turbo-charged L<TOML|https://toml.io> parsing!
+
+=begin html
+
+<a href='https://coveralls.io/github/FGasper/p5-TOML-XS?branch=master'><img src='https://coveralls.io/repos/github/FGasper/p5-TOML-XS/badge.svg?branch=master' alt='Coverage Status' /></a>
+
+=end html
 
 =head1 SYNOPSIS
 
@@ -30,7 +36,8 @@ TOML::XS - Parse L<TOML|https://toml.io> with XS
 =head1 DESCRIPTION
 
 This module facilitates parsing of TOML documents in Perl via XS,
-which should offer much greater speed than pure-Perl TOML parsers.
+which can yield dramatic performance gains relative to pure-Perl TOML
+libraries.
 
 It is currently implemented as a wrapper around the
 L<tomlc99|https://github.com/cktan/tomlc99> C library.
@@ -42,6 +49,11 @@ L<tomlc99|https://github.com/cktan/tomlc99> C library.
 Converts a byte string (i.e., raw, undecoded bytes) that contains a
 serialized TOML document to a L<TOML::XS::Document> instance.
 
+Throws a suitable exception if the TOML document is unparseable. This
+doesn’t necessarily mean that I<any> malformed TOML content triggers such
+an error; for example, if an individual data item is malformed but the
+document is otherwise valid, this may return successful.
+
 =head1 MAPPING TOML TO PERL
 
 Most TOML data items map naturally to Perl. The following details
@@ -61,10 +73,27 @@ L<Types::Serialiser>.
 
 =head1 NOTE ON CHARACTER DECODING
 
-This library mimics the default configuration of popular JSON modules:
+This library mimics the default behaviour of popular JSON modules:
 the TOML input to the parser is expected to be a byte string, while the
 strings that the parser outputs are character strings.
 
+=head1 PERFORMANCE
+
+On my system the included (I<very> simple!) benchmark outputs:
+
+    Including TOML::Tiny …
+
+    small …
+                Rate toml_tiny   toml_xs
+    toml_tiny  1009/s        --      -95%
+    toml_xs   21721/s     2053%        --
+
+    large …
+                (warning: too few iterations for a reliable count)
+              s/iter toml_tiny   toml_xs
+    toml_tiny   1.65        --      -93%
+    toml_xs    0.110     1400%        --
+
 =cut
 
 #----------------------------------------------------------------------
@@ -72,14 +101,17 @@ strings that the parser outputs are character strings.
 *true = *Types::Serialiser::true;
 *false = *Types::Serialiser::false;
 
+sub _croak_err_message_from_pieces {
+    my $path_ar = shift;
+
+    s<~><~0>g for @$path_ar;
+    s</><~1>g for @$path_ar;
+
+    die sprintf "Malformed TOML value found! (JSON pointer: %s)", join('/', q<>, @$path_ar);
+}
+
 #----------------------------------------------------------------------
 
-=head1 PERFORMANCE
-
-For small- and medium-sized files this should be quite a bit faster
-than pure-Perl TOML parsers. With larger files the speed gains are more
-muted or even (??) reversed. (That’s an underlying issue with tomlc99.)
-
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2021 Gasper Software Consulting. All rights reserved.
